#include "mainwindow.h"

#include <QTouchEvent>
#include <QDebug>

 MainWindow::MainWindow(QGraphicsScene *scene, QWidget *parent)
     : QGraphicsView(scene, parent), totalScaleFactor(1)
 {
     viewport()->setAttribute(Qt::WA_AcceptTouchEvents);
     setTransformationAnchor(QGraphicsView::NoAnchor);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 }

 bool MainWindow::viewportEvent(QEvent *event)
 {
     QTouchEvent *touchEvent;

     switch (event->type()) {
     case QEvent::TouchBegin:
     case QEvent::TouchUpdate:
     case QEvent::TouchEnd:
         //Ignore touch on items
         QGraphicsView::viewportEvent(event);
         if(!event->isAccepted())
             return true;

         touchEvent = static_cast<QTouchEvent *>(event);
         if(touchEvent->touchPoints().count() == 1) {
             const QTouchEvent::TouchPoint &touchPoint1 = touchEvent->touchPoints().first();
             QPointF dTrans = touchPoint1.scenePos() - touchPoint1.lastScenePos();
             translate(dTrans.x(), dTrans.y());
         }else{
             const QTouchEvent::TouchPoint &touchPoint1 = touchEvent->touchPoints().first();
             const QTouchEvent::TouchPoint &touchPoint2 = touchEvent->touchPoints().last();
             QLineF line1(touchPoint1.lastScenePos(), touchPoint2.lastScenePos());
             QLineF line2(touchPoint1.scenePos(), touchPoint2.scenePos());

             float dAngle = line2.angleTo(line1);
             float dScale = line2.length() / line1.length();

             //Position de l'origine de l'image dans le repere 0
             QPointF origine0;
             //1) Determination du repere 1 associe au segment [lastA, lastB]
             //1.1) Calcul de la matrice de passage du repere 0 au repere 1
             QTransform m01(line1.dx(), line1.dy(),
                            -line1.dy(), line1.dx(),
                            touchPoint1.lastScenePos().x(), touchPoint1.lastScenePos().y());
             //1.2) Calcul de origine0 dans repere 1
             QTransform m10 = m01.inverted();
             QPointF origine1 = m10.map(origine0);

             //2) Calcul du repere 2 qui est celui associe au segment [currentA, currentB]
             QTransform m02(line2.dx(), line2.dy(),
                            -line2.dy(), line2.dx(),
                            touchPoint1.scenePos().x(), touchPoint1.scenePos().y());
             //La position de l'origine est la meme dans le repere 2 que celle dans le repere 1 au changement d'echelle pres.
             //m02 = m02.scale(dScale, dScale);  //pris en compte dans la construction de m02
             QPointF origine2 = origine1;
             QPointF origine0b = m02.map(origine2);
             QPointF dTrans = origine0b - origine0;  //Unnatural result

             translate(dTrans.x(), dTrans.y());
             rotate(dAngle);
             scale(dScale, dScale);

             //Limit zoom
             totalScaleFactor *= dScale;
             if(totalScaleFactor < 1){
                 scale(1/totalScaleFactor, 1/totalScaleFactor);
                 totalScaleFactor = 1;
             }else if(totalScaleFactor > 200){
                 scale(200/totalScaleFactor, 200/totalScaleFactor);
                 totalScaleFactor = 200;
             }
         }
         break;

     default:
         return QGraphicsView::viewportEvent(event);
     }
     return true;
}

 void MainWindow::keyPressEvent(QKeyEvent* event){
     //keyboard event, application control
     switch(event->key()){
     case Qt::Key_Escape:
         close();
         break;

     default:
         QWidget::keyPressEvent(event);
         break;
     }
 }
