/*
    qTUIO - TUIO Interface for Qt

    Original Version by Martin Blankenburg <martin.blankenburg@imis.uni-luebeck.de>
    Integrated into qTUIO by x29a <0.x29a.0@gmail.com>
    Some modifications by Paolo Olivo <olivopaolo@tiscali.it>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef QTUIO_H
#define QTUIO_H

#include "qtuio_dll.h"

#include <QThread>
#include <QMap>
#include <QEvent>
#include <QTouchEvent>
#include <QGenericMatrix>
#include "./3rdparty/tuio/TuioListener.h"

namespace TUIO {
    class TuioClient;
    class TuioCursor;
}

class QGraphicsView;
class QGraphicsScene;
class QMainWindow;
class QRect;
class QGraphicsObject;
class SpeedListener;

class QTUIO_DLLSPEC QTuio : public QThread, public TUIO::TuioListener {
    Q_OBJECT

public:
    explicit QTuio(QObject *parent = 0);
    ~QTuio();

    void run();

    void setTuioCalibration(SpeedListener *widget, QMatrix3x3 &calibration) ;
    void associateSourceWidget(QString name, SpeedListener *widget);

    virtual void addTuioObject(TUIO::TuioObject *tobj);
    virtual void updateTuioObject(TUIO::TuioObject *tobj);
    virtual void removeTuioObject(TUIO::TuioObject *tobj);
    virtual void addTuioBlob(TUIO::TuioBlob *tblb);
    virtual void updateTuioBlob(TUIO::TuioBlob *tblb);
    virtual void removeTuioBlob(TUIO::TuioBlob *tblb);
    virtual void addTuioCursor(TUIO::TuioCursor *tcur);
    virtual void updateTuioCursor(TUIO::TuioCursor *tcur);
    virtual void removeTuioCursor(TUIO::TuioCursor *tcur);
    virtual void refresh(TUIO::TuioTime ftime);

public slots:
    void dissociateWidget(QString name);

signals:
    void newSource(QString name, QString ipaddr);

private:
    struct Tuio2Qt {
        QTouchEvent::TouchPoint tPoint;
        QWidget *widget;
    };
    bool running;
    int id;
    QGraphicsScene *theScene;
    QGraphicsView *theView;
    QWidget *theWidget;
    /** Calibration matrix to convert TUIO [0,1] coordinates to screen
	coordinates. */
    QMap<QString, QMatrix3x3> calibration;
    TUIO::TuioClient *tuioClient;
    QMap<QString, SpeedListener*> namesSource;
    QMap<QString, Tuio2Qt> tuio2Qt;
    QMap<QWidget *, QList<QString> > widgets;

    bool tuioToQt(TUIO::TuioCursor *tcur, QEvent::Type eventType);
    /** Update the TouchPoint correspondent to the argument
      TuioCursor.  This methos exclusively updates the position
      variables. */
    void updateTouch(TUIO::TuioCursor *tcur);
    QPoint tuioToScreen(QString name, const QPointF &norm) const;
    QPointF mapWidgetToGlobal(QWidget *widget, const QPointF &pos) const;
};

#endif // QTUIO_H
