#include "abstractwidget.h"

#include "ghostview.h"
#include "cursor.h"

#define ADJUST 10

AbstractWidget::AbstractWidget(GhostView *g, QColor col) : ghostview(g)
{
    setAcceptTouchEvents(true);
    setOpacity(0.6);
    x_width = 400;
    y_height = x_width * 9/16.0;

    color = col;

    memoryLeftClick = -1;
    memoryRightClick = -1;
}

AbstractWidget::~AbstractWidget(){
    myDestroyed(name);
}

void AbstractWidget::setParameters(float px2mmScreen, QSizeF sizeTouchpad){
    this->px2mmScreen = px2mmScreen;
    this->sizeTouchpad = sizeTouchpad;
}

QRectF AbstractWidget::boundingRect() const
{
    return QRectF(-x_width/2.0 - ADJUST, -y_height/2.0 - ADJUST,
                   x_width + 2*ADJUST,    y_height + 2*ADJUST);
}

void AbstractWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget){
    //Call Paint method of under-class
    underPaint(painter, option, widget);
}

int AbstractWidget::createCursor(QPointF pos){
    Cursor *cursor = new Cursor(ghostview);
    cursor->setColor(color);
    ghostview->scene()->addItem(cursor);
    cursor->setPos(pos);
    cursor->setRotation(parentItem()->rotation());
    cursor->setZValue(0.9);
    cursor->setLeftClick(false);
    connect(this, SIGNAL(destroyed()), cursor, SLOT(deleteLater()));

    cursors.insert(cursor->getCursorNb(), cursor);
    return cursor->getCursorNb();
}

void AbstractWidget::frameRotate(){
    foreach(Cursor *c, cursors)
        c->setRotation(parentItem()->rotation());
}

QList<QTouchEvent::TouchPoint> AbstractWidget::getTouchpointList(){
    return listPtsOnWidget.values();
}
