/*
 * Cursor - Touch cursor
 *
 * Author: Jérémie Gilliot <jeremie.gilliot@inria.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CURSOR_H
#define CURSOR_H

#include <QGraphicsObject>
#include <QGraphicsView>
#include <QWidget>
#include <QMap>
#include <QTouchEvent>

#include "ghostview.h"


class Cursor : public QGraphicsObject
{
    Q_OBJECT
private:
    GhostView *ghostview;

    QColor color;

    bool leftClick, oldLeftClick;

    int cursorNb;
    bool centerClick;  //used for handCursor to stock center state : if no cursel for example
    int eventCenterId;

    QPointF firstTouch;
    int firstId;

protected:
    static int takeNumber();
    static void releaseNumber(int id);

    void updateTouch(QTouchEvent::TouchPoint *newTouchPoint);
    struct touchWidget{
        QTouchEvent::TouchPoint tPoint;
        QWidget *widget;
    };
    QMap<int, struct touchWidget> touchPointList;

    QMap<int, QPointF> curselList;

    QList<QPointF> oldCurselList;  //used for update problems
    QPointF oldPos;

public:
    Cursor(GhostView *g);
    virtual ~Cursor();

    void setColor(QColor col) { color = col; color.setAlpha(150); }

    int getCursorNb();
    bool isIdPartOfCursor(int id);

    QRectF boundingRect() const;
    QPainterPath shape() const;
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
    bool sceneEvent(QEvent *event);
    void sendUpdateEvent();

    void generateEvent();

    void setLeftClick(bool c);
    bool getLeftClick(){ return leftClick; }

    int addCursel(QPointF fPos);
    void moveCursel(int id, QPointF fPos);
    void removeCursel(int id, QPointF fPos);

    int getFirstId(){ return firstId; }

signals:
    void receiveTouchBegin();
    void receiveTouchEnd();
};

#endif // CURSOR_H
