#include "feedbackcreation.h"

#include "cursor.h"

#include <math.h>

#ifndef M_PI
#define M_PI       3.14159265358979323846
#define M_PI_2     1.57079632679489661923
#endif

#include <QApplication>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QPainter>
#include <QStyleOption>
#include <QTouchEvent>
#include <QPropertyAnimation>

#include "ghostview.h"

#define ADJUST 10

#define LENGTH_MIN (GhostView::scaleFactor*50/GhostView::px2mmScreen)

FeedbackCreation::FeedbackCreation() : QGraphicsObject()
{
    setVisible(false);
    setOpacity(0.2);

    valid = false;
    stickyCursorValue = 0;
}

QRectF FeedbackCreation::boundingRect() const
{
    float cursorY = mapFromScene(cursorPos).y();
    float x = -GhostView::circleSize;
    float y;
    if(abs(cursorY) < LENGTH_MIN)
        y = LENGTH_MIN;
    else
        y = cursorY;
    QRectF rect(x, GhostView::circleSize, 2*GhostView::circleSize, -abs(y)-2*GhostView::circleSize);
    return rect.adjusted(-ADJUST, ADJUST, ADJUST, -ADJUST);
}

QPainterPath FeedbackCreation::shape() const
{
    QPainterPath path;
    return path;
}

void FeedbackCreation::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    setPos(controlBegin);
    QLineF line1(QPointF(0,0) , QPointF(0,-100));
    QLineF line2(QPointF(0,0) , mapFromScene(cursorBegin));
    setRotation(-line1.angleTo(line2)+rotation());

    QPen pen;
    pen.setColor(color);
    pen.setWidth(1/GhostView::px2mmScreen);
    painter->setPen(pen);

    float cursorY = mapFromScene(cursorPos).y();

    QPainterPath path;
    float factor = (1-(2*GhostView::circleSize/-cursorY))*M_PI_2;
    path.setFillRule(Qt::WindingFill);

    if(stickyCursorValue*cursorY+GhostView::circleSize < 0){
        path.moveTo(2, 0);
        path.arcTo(-2, 0, 4,4, 0,-180);
        path.quadTo(-GhostView::circleSize*pow(cos(M_PI/2.5),2), (stickyCursorValue*cursorY+GhostView::circleSize),
                     -GhostView::circleSize*cos(M_PI/2.5), (stickyCursorValue*cursorY+GhostView::circleSize*sin(M_PI/2.5)));
        path.lineTo(GhostView::circleSize*cos(M_PI/2.5), (stickyCursorValue*cursorY+GhostView::circleSize*sin(M_PI/2.5)));
        path.quadTo(GhostView::circleSize*pow(cos(M_PI/2.5),2), (stickyCursorValue*cursorY+GhostView::circleSize),
                     2,0);
    }

    path.moveTo(-GhostView::circleSize*cos(factor), stickyCursorValue*cursorY-GhostView::circleSize*sin(factor));
    path.cubicTo(-GhostView::circleSize*pow(cos(factor),2), stickyCursorValue*cursorY-GhostView::circleSize,
                 -GhostView::circleSize*pow(cos(factor),2), cursorY+GhostView::circleSize,
                 -GhostView::circleSize*cos(factor), cursorY+GhostView::circleSize*sin(factor));
    path.lineTo(GhostView::circleSize*cos(factor), cursorY+GhostView::circleSize*sin(factor));
    path.cubicTo(GhostView::circleSize*pow(cos(factor),2), cursorY+GhostView::circleSize,
                 GhostView::circleSize*pow(cos(factor),2), stickyCursorValue*cursorY-GhostView::circleSize,
                 GhostView::circleSize*cos(factor), stickyCursorValue*cursorY-GhostView::circleSize*sin(factor));
    path.lineTo(-GhostView::circleSize*cos(factor), stickyCursorValue*cursorY-GhostView::circleSize*sin(factor));

    if(cursorY > -LENGTH_MIN){
        path.moveTo(-GhostView::circleSize*cos(M_PI/2.5), cursorY-GhostView::circleSize*sin(M_PI/2.5));
        path.quadTo(-GhostView::circleSize*pow(cos(M_PI/2.5),2), cursorY-GhostView::circleSize,
                     -2, -LENGTH_MIN-GhostView::circleSize);
        path.arcTo(-2, -LENGTH_MIN-GhostView::circleSize-2, 4,4, 180,-180);
        path.quadTo(GhostView::circleSize*pow(cos(M_PI/2.5),2), cursorY-GhostView::circleSize,
                    GhostView::circleSize*cos(M_PI/2.5), cursorY-GhostView::circleSize*sin(M_PI/2.5));
        path.lineTo(-GhostView::circleSize*cos(M_PI/2.5), cursorY-GhostView::circleSize*sin(M_PI/2.5));
    }

    path.addEllipse(QPointF(0,stickyCursorValue*cursorY), GhostView::circleSize, GhostView::circleSize);
    path.addEllipse(QPointF(0,cursorY), GhostView::circleSize, GhostView::circleSize);

    painter->fillPath(path, color);
}


void FeedbackCreation::beginFingers(QPointF cursor, QPointF widget){
    cursorPos = cursor;
    controlPos = widget;
    cursorBegin = cursor;
    controlBegin = widget;

    setPos(controlBegin);
    QLineF line1(QPointF(0,0) , QPointF(0,-100));
    QLineF line2(QPointF(0,0) , mapFromScene(cursorBegin));
    setRotation(-line1.angleTo(line2)+rotation());

    setVisible(true);
}

void FeedbackCreation::updateControl(QPointF widget){
    controlPos = widget;
    if(QLineF(controlBegin, controlPos).length() > GhostView::circleSize){
        //qDebug("ref displacement");
        gestureCancelled();
        setVisible(false);
    }
    update();
}

void FeedbackCreation::updateCursor(QPointF cursor){
    cursorPos = cursor;
    //if(QLineF(controlBegin, cursorPos).length() < 3*GhostView::circleSize)
        cursorBegin = cursor;

    float u = ( (cursorPos.x()-controlBegin.x())*(cursorBegin.x()-controlBegin.x())
              + (cursorPos.y()-controlBegin.y())*(cursorBegin.y()-controlBegin.y()) )
              / pow(QLineF(controlBegin, cursorBegin).length(),2);
    QPointF proj = controlBegin + u*(cursorBegin-controlBegin);

    //if cursor pass the min length
    if( !valid
     && QLineF(cursorPos, proj).length() < GhostView::circleSize
     && QLineF(controlBegin, cursorPos).length() > LENGTH_MIN ){
        valid = true;
        //qDebug("anim deploy");
        QPropertyAnimation *animation = new QPropertyAnimation(this, "stickyCursorAnim");
        animation->setDuration(500);
        animation->setStartValue(0);
        animation->setEndValue(1);
        animation->setEasingCurve(QEasingCurve::OutCubic);
        animation->start(QAbstractAnimation::DeleteWhenStopped);

    //if finger quit cursor or cursor pass below the min length
    }else if( valid
           && (QLineF(cursorPos, proj).length() > GhostView::circleSize
              || QLineF(controlBegin, cursorPos).length() < LENGTH_MIN) ){
        valid = false;
        //qDebug("anim reploy");
        QPropertyAnimation *animation = new QPropertyAnimation(this, "stickyCursorAnim");
        animation->setDuration(500);
        animation->setStartValue(1);
        animation->setEndValue(0);
        animation->setEasingCurve(QEasingCurve::OutCubic);
        animation->start(QAbstractAnimation::DeleteWhenStopped);
    }
    update();
}

void FeedbackCreation::releaseControl(QPointF widget){
    gestureCancelled();
    setVisible(false);
    stickyCursorValue = 0;
    valid = false;
}

void FeedbackCreation::releaseCursor(QPointF cursor){
    if(valid){
        gesturePerformed(controlPos, cursorPos);
        gestureCancelled();
        setVisible(false);
        stickyCursorValue = 0;
        valid = false;
    }else{
        //gestureCancelled();
        if(valid){
            QPropertyAnimation *animation = new QPropertyAnimation(this, "stickyCursorAnim");
            animation->setDuration(500);
            animation->setStartValue(1);
            animation->setEndValue(0);
            animation->setEasingCurve(QEasingCurve::OutCubic);
            animation->start(QAbstractAnimation::DeleteWhenStopped);
        }

        QPropertyAnimation *animation = new QPropertyAnimation(this, "cursorPosAnim");
        animation->setDuration(200);
        animation->setStartValue(cursorPos);
        animation->setEndValue(controlPos);
        animation->setEasingCurve(QEasingCurve::OutCubic);
        animation->start(QAbstractAnimation::DeleteWhenStopped);
    }
}
