#include "keyboardwidget.h"

#include <QPropertyAnimation>
#include "keyrect.h"
#include "mingetframe.h"

#define THICKNESS (GhostView::scaleFactor/GhostView::px2mmScreen)


KeyboardWidget::KeyboardWidget(GhostView *g, QPointF widgetPos, float px2mmScreen) : AbstractWidget(g, QColor(150,150,150))
{
    upper = false;
    int keySize = 18;
    KeyRect::resetKeyIndex();
    QSize dim(14*keySize, 5*keySize);
    dim /= px2mmScreen;
    this->px2mmScreen = px2mmScreen;

    MingetFrame *frame = new MingetFrame(this, dim);
    ghostview->scene()->addItem(frame);
    frame->setPos(widgetPos);

    frame->setZValue(1);
    this->setParentItem(frame);
    setZValue(1.1);

    float stepW = dim.width()/14.0;
    float stepH = dim.height()/5.0;
    for(int y=0; y<5; y++){
        for(int x=0; x<14; x++){
            if(x==0&&y==2 || x==13&&y==2)  //shift
                new KeyRect(x*stepW-dim.width()/2.0, y*stepH-dim.height()/2.0, stepW, 2*stepH, this);
            else if(x==6&&y==2)  //enter
                new KeyRect(x*stepW-dim.width()/2.0, y*stepH-dim.height()/2.0, stepW, 2*stepH, this);
            else if(x==4&&y==4)  //space
                new KeyRect(x*stepW-dim.width()/2.0, y*stepH-dim.height()/2.0, 5*stepW, stepH, this);
            else if(x==0&&y==3 || x==13&&y==3 || x==6&&y==3 || x==5&&y==4|| x==6&&y==4|| x==7&&y==4|| x==8&&y==4)
                x;  //do nothing
            else if(y==4&&(x==10||x==11||x==12)){
                new KeyRect(x*stepW-dim.width()/2.0, y*stepH-dim.height()/2.0, stepW, stepH/2.0, this);
                new KeyRect(x*stepW-dim.width()/2.0, y*stepH-dim.height()/2.0+stepH/2.0, stepW, stepH/2.0, this);
            }else
                new KeyRect(x*stepW-dim.width()/2.0, y*stepH-dim.height()/2.0, stepW, stepH, this);
        }
    }

    connect(frame, SIGNAL(rotationChanged()), this, SLOT(frameRotate()));

    QPropertyAnimation *animation = new QPropertyAnimation(frame, "scale");
    animation->setDuration(1000);
    animation->setStartValue(0);
    animation->setEndValue(1);
    animation->setEasingCurve(QEasingCurve::OutBack);
    animation->start(QAbstractAnimation::DeleteWhenStopped);

    createCursor(widgetPos - QPoint(0, dim.height()/2.0));
}

QPainterPath KeyboardWidget::shape() const
{
    QPainterPath path;
    path.addRect(-x_width/2.0, -y_height/2.0, x_width, y_height);
    return path;
}

void KeyboardWidget::underPaint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    QPen pen;
    pen.setBrush(QColor(100, 100, 100));
    pen.setWidth(THICKNESS);
    painter->setPen(pen);

    //Touch surface
    //Keyboard surface
        painter->fillRect(-x_width/2, -y_height/2, x_width, y_height, color);
        /*painter->drawLine(-x_width/2, 0, x_width/2, 0);
        painter->drawLine(0, -y_height/2, 0, y_height/2);*/
}

bool KeyboardWidget::sceneEvent(QEvent *event)
{
    QTouchEvent *touchEvent;

    switch(event->type()){
    case QEvent::TouchBegin:
    case QEvent::TouchUpdate:
    case QEvent::TouchEnd:
        touchEvent = static_cast<QTouchEvent *>(event);
        //qDebug("event(%d) received %d touch", event->type(), touchEvent->touchPoints().size());
        foreach(QTouchEvent::TouchPoint touchPoint1, touchEvent->touchPoints()){
            switch(touchPoint1.state()){
            case Qt::TouchPointPressed:
            {
                //qDebug("  Pressed %d", touchPoint1.id());
                break;
            }

            case Qt::TouchPointMoved:
            //case Qt::TouchPointStationary:
            {
                //qDebug("  Moved %d", touchPoint1.id());
                break;
            }

            case Qt::TouchPointReleased:
            {
                //qDebug("  Released %d", touchPoint1.id());
                break;
            }

            default:
                break;
            }
        }
        break;

    default:
        return QGraphicsItem::sceneEvent(event);
    }
    update();
    return true;
}
