#include "keyrect.h"

#include <QGraphicsRectItem>
#include <QTouchEvent>
#include <QPainter>

/*#include <QDebug>
#include <QDesktopWidget>
#include <QApplication>
#include <QGraphicsWidget>*/

#include "keyboardwidget.h"

#ifdef WIN32
#include "Windows.h"
#include "winuser.h"
#endif

int KeyRect::index;
char* KeyRect::keys[14*5+3] = {"","&","","\"","'","(","suppr","-","","_","","",")","=",
                               "tab","a","z","e","r","t","back","y","u","i","o","p","^","$",
                               "shift","q","s","d","f","g","enter","h","j","k","l","m","","shift",
                               "w","x","c","v","b","n",",",";",":","!","*",
                               "ctrl","<","Meta","alt"," ","altGr",
                                                                      "home","<-","^","v","end","->","ctrl"};

char* KeyRect::upkeys[14*5+3] = {"","1","2","3","4","5","suppr","6","7","8","9","0","","+",
                               "tab","A","Z","E","R","T","back","Y","U","I","O","P","","",
                               "shift","Q","S","D","F","G","enter","H","J","K","L","M","%","shift",
                               "W","X","C","V","B","N","?",".","/","","",
                               "ctrl",">","Meta","alt"," ","altGr",
                                                                      "home","<-","^","v","end","->","ctrl"};

#ifdef WIN32
int KeyRect::vkcodes[14*5+3] = {VK_OEM_7,0x31,0x32,0x33,0x34,0x35,VK_DELETE,0x36,0x37,0x38,0x39,0x30,VK_OEM_4,VK_OEM_PLUS,
                        VK_TAB,0x41,0x5A,0x45,0x52,0x54,VK_BACK,0x59,0x55,0x49,0x4F,0x50,VK_OEM_6,VK_OEM_1,
                        VK_LSHIFT,0x51,0x53,0x44,0x46,0x47,VK_RETURN,0x48,0x4A,0x4B,0x4C,0x4D,VK_OEM_3,VK_RSHIFT,
                        0x57,0x58,0x43,0x56,0x42,0x4E,VK_OEM_COMMA,VK_OEM_PERIOD,VK_OEM_2,VK_OEM_8,VK_OEM_5,
                        VK_LCONTROL,VK_OEM_102,VK_LWIN,VK_LMENU,VK_SPACE,VK_RMENU,
                                                                        VK_HOME,VK_LEFT,VK_UP,VK_DOWN,VK_END,VK_RIGHT,VK_RCONTROL};
#endif

KeyRect::KeyRect(qreal x, qreal y, qreal w, qreal h, KeyboardWidget *parent) : QGraphicsRectItem(-w/2.0, -h/2.0, w, h, parent), keyboard(parent)
{
    key = keys[index];
    upkey = upkeys[index];
#ifdef WIN32
    vkcode = vkcodes[index];
#endif
    index++;
    setAcceptTouchEvents(true);
    setPos(x+w/2.0,y+h/2.0);
    touched = false;
}

void KeyRect::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget){
    if(touched)
        painter->fillRect(rect(), QColor(100,100,100));
    painter->setFont(QFont("MS Shell", 12));
    if(keyboard->isUpper())
        painter->drawText(rect(), Qt::AlignCenter, upkey);
    else
        painter->drawText(rect(), Qt::AlignCenter, key);
    QGraphicsRectItem::paint(painter, option, widget);
}

bool KeyRect::sceneEvent(QEvent *event){
    QTouchEvent *tEvent;
    #ifdef WIN32
    /*QPointF fp;
    QDesktopWidget qDesktopWidget;
    POINT pt;
    HWND handle;*/
    INPUT ip;
    ip.type = INPUT_KEYBOARD;
    ip.ki.wScan = 0; // hardware scan code for key
    ip.ki.time = 0;
    ip.ki.dwExtraInfo = 0;
    ip.ki.wVk = vkcode;
    #endif

    switch(event->type()){
    case QEvent::TouchBegin:
    case QEvent::TouchUpdate:
    case QEvent::TouchEnd:
        tEvent = static_cast<QTouchEvent *>(event);
        //foreach(QTouchEvent::TouchPoint touchPoint1, tEvent->touchPoints()){
        {QTouchEvent::TouchPoint touchPoint1 = tEvent->touchPoints()[0];
            switch(touchPoint1.state()){
            case Qt::TouchPointPressed:
                touched = true;
                if(strcmp(key, "shift")==0){
                    keyboard->setUpper();
                    keyboard->update();
                }
                #ifdef WIN32
                //attach a cursor to allow multi-keyboard and -focus
                /*fp = qDesktopWidget.mapToGlobal(keyboard->getFocusPos().toPoint()) + qDesktopWidget.pos();
                pt.x = fp.x(); pt.y = fp.y();
                handle = ChildWindowFromPoint(pt);
                SetForegroundWindow(handle);*/
                ip.ki.dwFlags = 0; // 0 for key press
                SendInput(1, &ip, sizeof(INPUT));
                #endif
                break;

            case Qt::TouchPointReleased:
                touched = false;
                if(strcmp(key, "shift")==0){
                    keyboard->setLower();
                    keyboard->update();
                }
                #ifdef WIN32
                //attach a cursor to allow multi-keyboard and -focus
                /*fp = qDesktopWidget.mapToGlobal(keyboard->getFocusPos().toPoint()) + qDesktopWidget.pos();
                pt.x = fp.x(); pt.y = fp.y();
                handle = ChildWindowFromPoint(pt);
                SetForegroundWindow(handle);*/
                ip.ki.dwFlags = KEYEVENTF_KEYUP; // KEYEVENTF_KEYUP for key release
                SendInput(1, &ip, sizeof(INPUT));
                #endif
                break;

            default:
                break;
            }
            update();
        }

        //Accept all touch events received
        return true;

    default:
        break;
    }
    return QGraphicsRectItem::sceneEvent(event);
}
