#include "mingetsettings.h"

#include <QApplication>
#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QPainter>
#include <QStyleOption>
#include <QTouchEvent>

#include "mingetframe.h"
#include "ghostview.h"

#define ADJUST 10
#define BUTTON_SIZE (GhostView::scaleFactor*3/GhostView::px2mmScreen)

MingetSettings::MingetSettings(MingetFrame *f) : QGraphicsObject(), frame(f)
{
    updatePos();
    connect(frame, SIGNAL(heightChanged()), this, SLOT(updatePos()));
    connect(frame, SIGNAL(widthChanged()), this, SLOT(updatePos()));
    setAcceptTouchEvents(true);
}

QRectF MingetSettings::boundingRect() const
{
    return QRectF(-BUTTON_SIZE - ADJUST, -BUTTON_SIZE - ADJUST,
                  2*BUTTON_SIZE + 2*ADJUST, 2*BUTTON_SIZE + 2*ADJUST);
}

QPainterPath MingetSettings::shape() const
{
    QPainterPath path;
    path.addEllipse(-BUTTON_SIZE, -BUTTON_SIZE, BUTTON_SIZE * 2, BUTTON_SIZE * 2);
    return path;
}

void MingetSettings::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    float thic = (GhostView::scaleFactor*1/GhostView::px2mmScreen);
    QPainterPath path;
    path.addEllipse(QPoint(BUTTON_SIZE-1.5*thic, 0), 1.5*thic, 1.5*thic);
    path.addRect(-BUTTON_SIZE, -thic/2.0, 2*(BUTTON_SIZE), thic);
    path.addRect(BUTTON_SIZE-3*thic, -thic/2.0, 1.5*thic, thic);
    painter->rotate(-45);
    painter->fillPath(shape().subtracted(path), QColor(100, 100, 100));
}

bool MingetSettings::sceneEvent(QEvent *event)
{
    QTouchEvent *touchEvent;

    switch (event->type()) {
    case QEvent::TouchBegin:
        return true;
    case QEvent::TouchUpdate:
        return true;
    case QEvent::TouchEnd:
        touchEvent = static_cast<QTouchEvent *>(event);
        if(!touchEvent->touchPoints().isEmpty() && contains(touchEvent->touchPoints().first().pos()))
            frame->toggleSettings();
        return true;

    default:
        return QGraphicsItem::sceneEvent(event);
    }
}

void MingetSettings::updatePos(){
    setPos(frame->boundingRect().width()/2.0-ADJUST-2*BUTTON_SIZE, -frame->boundingRect().height()/2.0+ADJUST+4.5*BUTTON_SIZE);
}
